define(["require", "exports", "tslib", "aurelia-pal", "aurelia-binding"], function (require, exports, tslib_1, aurelia_pal_1, aurelia_binding_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.MdcSelectValueObserver = void 0;
    /**
     * @hidden
     * The observer only passes values to the element if options has been set at least once.
     * When the options are set for the first time the observer passes the value to the element.
     * Subsequent options changes pull the value from the element.
     * Subsequent value changes are passed to the element.
     */
    var MdcSelectValueObserver = /** @class */ (function () {
        function MdcSelectValueObserver(element, handler) {
            this.handler = handler;
            this.element = element;
        }
        MdcSelectValueObserver.prototype.setElementValue = function (skipNotify) {
            if (skipNotify === void 0) { skipNotify = false; }
            // do not pass the value to the element if options has never been set
            // the value will be passed on when options do arrive
            if (this.optionsWereSet) {
                this.element.au.controller.viewModel.setValue(this.value, skipNotify);
            }
        };
        MdcSelectValueObserver.prototype.getValue = function () {
            return this.value;
        };
        MdcSelectValueObserver.prototype.setValue = function (newValue) {
            if (this.value === newValue) {
                return;
            }
            // assign and sync element.
            this.oldValue = this.value;
            this.value = newValue;
            this.setElementValue();
            this.notify();
        };
        MdcSelectValueObserver.prototype.synchronizeValue = function () {
            var value = this.element.value;
            if (value !== this.value) {
                this.oldValue = this.value;
                this.value = value;
                this.notify();
            }
        };
        MdcSelectValueObserver.prototype.notify = function () {
            var oldValue = this.oldValue;
            var newValue = this.value;
            this.callSubscribers(newValue, oldValue);
        };
        MdcSelectValueObserver.prototype.handleEvent = function () {
            this.synchronizeValue();
        };
        MdcSelectValueObserver.prototype.subscribe = function (context, callable) {
            if (!this.hasSubscribers()) {
                this.handler.subscribe(this.element, this);
            }
            this.addSubscriber(context, callable);
        };
        MdcSelectValueObserver.prototype.unsubscribe = function (context, callable) {
            if (this.removeSubscriber(context, callable) && !this.hasSubscribers()) {
                this.handler.dispose();
            }
        };
        MdcSelectValueObserver.prototype.bind = function () {
            var _this = this;
            var menu = this.element.querySelector('.mdc-menu');
            this.optionsWereSet = !!menu.querySelector('mdc-list-item');
            this.domObserver = aurelia_pal_1.DOM.createMutationObserver(function (records) {
                if (records.find(function (x) { return x.type === 'childList'
                    && (Array.from(x.addedNodes).find(function (y) { return y.tagName === 'MDC-LIST-ITEM'; })
                        || Array.from(x.removedNodes).find(function (y) { return y.tagName === 'MDC-LIST-ITEM'; })); })) {
                    if (_this.optionsWereSet) {
                        if (_this.element.value != _this.value) {
                            _this.element.value = undefined;
                        }
                    }
                    else {
                        _this.optionsWereSet = true;
                        // if options are set for the first time pass the current value to the element
                        _this.setElementValue(true);
                    }
                    _this.synchronizeValue();
                }
            });
            this.domObserver.observe(menu, { childList: true, subtree: true, characterData: true });
        };
        MdcSelectValueObserver.prototype.unbind = function () {
            var _a;
            this.optionsWereSet = false;
            (_a = this.domObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
            this.domObserver = null;
        };
        MdcSelectValueObserver = tslib_1.__decorate([
            (0, aurelia_binding_1.subscriberCollection)(),
            tslib_1.__metadata("design:paramtypes", [Element, aurelia_binding_1.EventSubscriber])
        ], MdcSelectValueObserver);
        return MdcSelectValueObserver;
    }());
    exports.MdcSelectValueObserver = MdcSelectValueObserver;
});
//# sourceMappingURL=mdc-select-value-observer.js.map