define(["require", "exports", "tslib", "@aurelia-mdc-web/base", "@material/select", "aurelia-framework", "aurelia-pal", "./mdc-select-icon", "./mdc-select-helper-text/mdc-select-helper-text", "aurelia-typed-observable-plugin", "./mdc-select-foundation-aurelia", "./mdc-default-select-configuration"], function (require, exports, tslib_1, base_1, select_1, aurelia_framework_1, aurelia_pal_1, mdc_select_icon_1, mdc_select_helper_text_1, aurelia_typed_observable_plugin_1, mdc_select_foundation_aurelia_1, mdc_default_select_configuration_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.MdcSelect = void 0;
    select_1.strings.CHANGE_EVENT = select_1.strings.CHANGE_EVENT.toLowerCase();
    var selectId = 0;
    /**
     * @selector mdc-select
     * @emits mdcselect:change | Emitted if user changed the value
     */
    var MdcSelect = /** @class */ (function (_super) {
        tslib_1.__extends(MdcSelect, _super);
        function MdcSelect(root, taskQueue, defaultConfiguration) {
            var _this = _super.call(this, root) || this;
            _this.taskQueue = taskQueue;
            _this.defaultConfiguration = defaultConfiguration;
            _this.id = "mdc-select-".concat(++selectId);
            _this.errors = new Map();
            /** Styles the select as an outlined select */
            _this.outlined = _this.defaultConfiguration.outlined;
            defineMdcSelectElementApis(_this.root);
            return _this;
        }
        MdcSelect.processContent = function (_viewCompiler, _resources, element) {
            // move icon to the slot - this allows omitting slot specification
            var leadingIcon = element.querySelector("[".concat(mdc_select_icon_1.mdcIconStrings.ATTRIBUTE, "]"));
            leadingIcon === null || leadingIcon === void 0 ? void 0 : leadingIcon.setAttribute('slot', 'leading-icon');
            return true;
        };
        MdcSelect.prototype.labelChanged = function () {
            var _this = this;
            this.taskQueue.queueTask(function () { var _a; return (_a = _this.foundation) === null || _a === void 0 ? void 0 : _a.layout(); });
        };
        MdcSelect.prototype.outlinedChanged = function () {
            var _this = this;
            this.taskQueue.queueTask(function () { var _a; return (_a = _this.foundation) === null || _a === void 0 ? void 0 : _a.layout(); });
        };
        MdcSelect.prototype.requiredChanged = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var _this = this;
                var _a, _b, _c;
                return tslib_1.__generator(this, function (_d) {
                    switch (_d.label) {
                        case 0: return [4 /*yield*/, this.initialised];
                        case 1:
                            _d.sent();
                            if (this.required) {
                                (_a = this.selectAnchor) === null || _a === void 0 ? void 0 : _a.setAttribute('aria-required', 'true');
                            }
                            else {
                                (_b = this.selectAnchor) === null || _b === void 0 ? void 0 : _b.removeAttribute('aria-required');
                            }
                            (_c = this.foundation) === null || _c === void 0 ? void 0 : _c.setRequired(this.required);
                            this.taskQueue.queueTask(function () { var _a; return (_a = _this.foundation) === null || _a === void 0 ? void 0 : _a.layout(); });
                            return [2 /*return*/];
                    }
                });
            });
        };
        MdcSelect.prototype.disabledChanged = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var _a;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0: return [4 /*yield*/, this.initialised];
                        case 1:
                            _b.sent();
                            (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.setDisabled(this.disabled);
                            return [2 /*return*/];
                    }
                });
            });
        };
        Object.defineProperty(MdcSelect.prototype, "value", {
            get: function () {
                if (this.foundation) {
                    return this.foundation.getValue();
                }
                else {
                    return this._value;
                }
            },
            set: function (value) {
                this.setValue(value);
            },
            enumerable: false,
            configurable: true
        });
        MdcSelect.prototype.setValue = function (value, skipNotify) {
            if (skipNotify === void 0) { skipNotify = false; }
            this._value = value;
            if (this.foundation) {
                this.foundation.setValue(value, skipNotify);
                this.foundation.layout();
            }
        };
        Object.defineProperty(MdcSelect.prototype, "valid", {
            get: function () {
                var _a, _b;
                return (_b = (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.isValid()) !== null && _b !== void 0 ? _b : true;
            },
            set: function (value) {
                var _a;
                (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.setValid(value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MdcSelect.prototype, "selectedIndex", {
            get: function () {
                return this.foundation.getSelectedIndex();
            },
            set: function (selectedIndex) {
                var _a;
                (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.setSelectedIndex(selectedIndex, /** closeMenu */ true);
            },
            enumerable: false,
            configurable: true
        });
        MdcSelect.prototype.addError = function (error) {
            this.errors.set(error, true);
            this.valid = false;
        };
        MdcSelect.prototype.removeError = function (error) {
            this.errors.delete(error);
            this.valid = this.errors.size === 0;
        };
        MdcSelect.prototype.renderErrors = function () {
            var helperText = this.root.nextElementSibling;
            if ((helperText === null || helperText === void 0 ? void 0 : helperText.tagName) === 'MDC-SELECT-HELPER-TEXT') {
                helperText.au.controller.viewModel.errors = Array.from(this.errors.keys())
                    .filter(function (x) { return x.message !== null; }).map(function (x) { return x.message; });
            }
        };
        MdcSelect.prototype.initialise = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var leadingIconEl, nextSibling;
                var _a;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            leadingIconEl = this.root.querySelector("[".concat(mdc_select_icon_1.mdcIconStrings.ATTRIBUTE, "]"));
                            this.leadingIcon = leadingIconEl === null || leadingIconEl === void 0 ? void 0 : leadingIconEl.au['mdc-select-icon'].viewModel;
                            nextSibling = this.root.nextElementSibling;
                            if ((nextSibling === null || nextSibling === void 0 ? void 0 : nextSibling.tagName) === mdc_select_helper_text_1.mdcHelperTextCssClasses.ROOT.toUpperCase()) {
                                this.helperText = nextSibling.au.controller.viewModel;
                            }
                            return [4 /*yield*/, Promise.all([(_a = this.helperText) === null || _a === void 0 ? void 0 : _a.initialised, this.menu.initialised].filter(function (x) { return x; }))];
                        case 1:
                            _b.sent();
                            this.menu.list_.singleSelection = true;
                            return [2 /*return*/];
                    }
                });
            });
        };
        MdcSelect.prototype.initialSyncWithDOM = function () {
            var _a, _b;
            // set initial value without emitting change events
            (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.setValue(this._value, true);
            (_b = this.foundation) === null || _b === void 0 ? void 0 : _b.layout();
            this.errors = new Map();
            this.valid = true;
        };
        MdcSelect.prototype.getDefaultFoundation = function () {
            // DO NOT INLINE this variable. For backward compatibility, foundations take a Partial<MDCFooAdapter>.
            // To ensure we don't accidentally omit any methods, we need a separate, strongly typed adapter variable.
            var adapter = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, this.getSelectAdapterMethods()), this.getCommonAdapterMethods()), this.getOutlineAdapterMethods()), this.getLabelAdapterMethods());
            return new mdc_select_foundation_aurelia_1.MDCSelectFoundationAurelia(adapter, this.getFoundationMap());
        };
        MdcSelect.prototype.getSelectAdapterMethods = function () {
            var _this = this;
            return {
                setSelectedText: function (text) {
                    _this.selectedText.textContent = text;
                },
                isSelectAnchorFocused: function () { return document.activeElement === _this.selectAnchor; },
                getSelectAnchorAttr: function (attr) { return _this.selectAnchor.getAttribute(attr); },
                setSelectAnchorAttr: function (attr, value) {
                    _this.selectAnchor.setAttribute(attr, value);
                },
                removeSelectAnchorAttr: function (attr) {
                    _this.selectAnchor.removeAttribute(attr);
                },
                addMenuClass: function (className) {
                    var _a;
                    (_a = _this.menuElement) === null || _a === void 0 ? void 0 : _a.classList.add(className);
                },
                removeMenuClass: function (className) {
                    var _a;
                    (_a = _this.menuElement) === null || _a === void 0 ? void 0 : _a.classList.remove(className);
                },
                openMenu: function () {
                    _this.menu.open = true;
                    _this.menu.root.style.minWidth = _this.menu.root.style.maxWidth = (_this.hoistToBody || _this.fixed) && !_this.naturalWidth
                        ? "".concat(_this.root.clientWidth, "px")
                        : '';
                },
                closeMenu: function () { _this.menu.open = false; },
                getAnchorElement: function () { return _this.root.querySelector(select_1.strings.SELECT_ANCHOR_SELECTOR); },
                setMenuAnchorElement: function (anchorEl) {
                    _this.menu.anchor = anchorEl;
                },
                setMenuAnchorCorner: function (anchorCorner) {
                    _this.menu.setAnchorCorner(anchorCorner);
                },
                setMenuWrapFocus: function (wrapFocus) {
                    _this.menu.wrapFocus = wrapFocus;
                },
                getSelectedIndex: function () {
                    var index = _this.menu.selectedIndex;
                    return index instanceof Array ? index[0] : index;
                },
                setSelectedIndex: function (index) {
                    _this.menu.selectedIndex = index;
                },
                removeAttributeAtIndex: function (index, attributeName) {
                    _this.menu.items[index].removeAttribute(attributeName);
                },
                focusMenuItemAtIndex: function (index) {
                    _this.menu.items[index].focus();
                },
                getMenuItemCount: function () { return _this.menu.items.length; },
                getMenuItemValues: function () { return _this.menu.items.map(function (x) { return x.au.controller.viewModel.value; }); },
                getMenuItemTextAtIndex: function (index) { return _this.menu.getPrimaryTextAtIndex(index); },
                isTypeaheadInProgress: function () { return _this.menu.typeaheadInProgress; },
                typeaheadMatchItem: function (nextChar, startingIndex) { return _this.menu.typeaheadMatchItem(nextChar, startingIndex); },
            };
        };
        MdcSelect.prototype.getCommonAdapterMethods = function () {
            var _this = this;
            return {
                addClass: function (className) {
                    _this.root.classList.add(className);
                },
                removeClass: function (className) {
                    _this.root.classList.remove(className);
                },
                hasClass: function (className) { return _this.root.classList.contains(className); },
                setRippleCenter: function (normalizedX) { var _a; return (_a = _this.lineRipple) === null || _a === void 0 ? void 0 : _a.setRippleCenter(normalizedX); },
                activateBottomLine: function () { var _a; return (_a = _this.lineRipple) === null || _a === void 0 ? void 0 : _a.activate(); },
                deactivateBottomLine: function () { var _a; return (_a = _this.lineRipple) === null || _a === void 0 ? void 0 : _a.deactivate(); },
                notifyChange: function (value) {
                    var index = _this.selectedIndex;
                    _this.emit(select_1.strings.CHANGE_EVENT, { value: value, index: index }, true /* shouldBubble  */);
                    _this.emit('change', { value: value, index: index }, true /* shouldBubble  */);
                },
            };
        };
        MdcSelect.prototype.getOutlineAdapterMethods = function () {
            var _this = this;
            return {
                hasOutline: function () { return Boolean(_this.outline); },
                notchOutline: function (labelWidth) { var _a; return (_a = _this.outline) === null || _a === void 0 ? void 0 : _a.notch(labelWidth); },
                closeOutline: function () { var _a; return (_a = _this.outline) === null || _a === void 0 ? void 0 : _a.closeNotch(); },
            };
        };
        MdcSelect.prototype.getLabelAdapterMethods = function () {
            var _this = this;
            return {
                hasLabel: function () { return !!_this.mdcLabel; },
                floatLabel: function (shouldFloat) { var _a; return (_a = _this.mdcLabel) === null || _a === void 0 ? void 0 : _a.float(shouldFloat); },
                getLabelWidth: function () { return _this.mdcLabel ? _this.mdcLabel.getWidth() : 0; },
                setLabelRequired: function (isRequired) { var _a; return (_a = _this.mdcLabel) === null || _a === void 0 ? void 0 : _a.setRequired(isRequired); },
            };
        };
        MdcSelect.prototype.handleChange = function () {
            var _a;
            (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleChange();
            this.emit('change', {}, true);
        };
        MdcSelect.prototype.handleFocus = function () {
            var _a;
            (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleFocus();
        };
        MdcSelect.prototype.handleBlur = function () {
            var _a;
            (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleBlur();
            // if class is set it means the menu is open,
            // do not emit blur since "conceptually" the element is still active
            if (!this.root.classList.contains(select_1.cssClasses.FOCUSED)) {
                this.emit('blur', {}, true);
            }
        };
        MdcSelect.prototype.handleClick = function (evt) {
            var _a;
            this.selectAnchor.focus();
            (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleClick(this.getNormalizedXCoordinate(evt));
        };
        MdcSelect.prototype.handleKeydown = function (evt) {
            var _a;
            (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleKeydown(evt);
            return true;
        };
        MdcSelect.prototype.handleMenuItemAction = function (evt) {
            var _a;
            (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleMenuItemAction(evt.detail.index);
        };
        MdcSelect.prototype.handleMenuOpened = function () {
            var _a;
            (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleMenuOpened();
        };
        MdcSelect.prototype.handleMenuClosed = function () {
            var _a;
            (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleMenuClosed();
            if (!this.root.classList.contains(select_1.cssClasses.FOCUSED)) {
                this.emit('blur', {}, true);
            }
        };
        MdcSelect.prototype.handleItemsChanged = function () {
            var _a, _b;
            (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.layoutOptions();
            (_b = this.foundation) === null || _b === void 0 ? void 0 : _b.layout();
        };
        MdcSelect.prototype.focus = function () {
            this.selectAnchor.focus();
        };
        MdcSelect.prototype.blur = function () {
            this.selectAnchor.blur();
        };
        /**
         * @hidden
         * Calculates where the line ripple should start based on the x coordinate within the component.
         */
        MdcSelect.prototype.getNormalizedXCoordinate = function (evt) {
            var targetClientRect = evt.target.getBoundingClientRect();
            var xCoordinate = this.isTouchEvent(evt) ? evt.touches[0].clientX : evt.clientX;
            return xCoordinate - targetClientRect.left;
        };
        MdcSelect.prototype.isTouchEvent = function (evt) {
            return Boolean(evt.touches);
        };
        /**
         * @hidden
         * Returns a map of all subcomponents to subfoundations.
         */
        MdcSelect.prototype.getFoundationMap = function () {
            var _a, _b;
            return {
                helperText: (_a = this.helperText) === null || _a === void 0 ? void 0 : _a.foundation,
                leadingIcon: (_b = this.leadingIcon) === null || _b === void 0 ? void 0 : _b.foundation
            };
        };
        tslib_1.__decorate([
            (0, aurelia_framework_1.children)('mdc-list-items'),
            tslib_1.__metadata("design:type", Array)
        ], MdcSelect.prototype, "items", void 0);
        tslib_1.__decorate([
            aurelia_typed_observable_plugin_1.bindable.none,
            tslib_1.__metadata("design:type", String)
        ], MdcSelect.prototype, "label", void 0);
        tslib_1.__decorate([
            aurelia_typed_observable_plugin_1.bindable.booleanAttr,
            tslib_1.__metadata("design:type", Boolean)
        ], MdcSelect.prototype, "outlined", void 0);
        tslib_1.__decorate([
            aurelia_typed_observable_plugin_1.bindable.booleanAttr,
            tslib_1.__metadata("design:type", Boolean)
        ], MdcSelect.prototype, "required", void 0);
        tslib_1.__decorate([
            aurelia_typed_observable_plugin_1.bindable.booleanAttr,
            tslib_1.__metadata("design:type", Boolean)
        ], MdcSelect.prototype, "disabled", void 0);
        tslib_1.__decorate([
            aurelia_typed_observable_plugin_1.bindable.booleanAttr({ defaultBindingMode: aurelia_framework_1.bindingMode.oneTime }),
            tslib_1.__metadata("design:type", Boolean)
        ], MdcSelect.prototype, "hoistToBody", void 0);
        tslib_1.__decorate([
            aurelia_typed_observable_plugin_1.bindable.booleanAttr({ defaultBindingMode: aurelia_framework_1.bindingMode.oneTime }),
            tslib_1.__metadata("design:type", Boolean)
        ], MdcSelect.prototype, "fixed", void 0);
        tslib_1.__decorate([
            aurelia_typed_observable_plugin_1.bindable.none,
            tslib_1.__metadata("design:type", Object)
        ], MdcSelect.prototype, "anchorMargin", void 0);
        tslib_1.__decorate([
            aurelia_typed_observable_plugin_1.bindable.booleanAttr,
            tslib_1.__metadata("design:type", Boolean)
        ], MdcSelect.prototype, "naturalWidth", void 0);
        MdcSelect = tslib_1.__decorate([
            (0, aurelia_framework_1.inject)(Element, aurelia_framework_1.TaskQueue, mdc_default_select_configuration_1.MdcDefaultSelectConfiguration),
            (0, aurelia_framework_1.useView)(aurelia_pal_1.PLATFORM.moduleName('./mdc-select.html')),
            (0, aurelia_framework_1.customElement)(select_1.cssClasses.ROOT),
            (0, aurelia_framework_1.processContent)(MdcSelect.processContent),
            tslib_1.__metadata("design:paramtypes", [HTMLElement, aurelia_framework_1.TaskQueue, mdc_default_select_configuration_1.MdcDefaultSelectConfiguration])
        ], MdcSelect);
        return MdcSelect;
    }(base_1.MdcComponent));
    exports.MdcSelect = MdcSelect;
    function defineMdcSelectElementApis(element) {
        Object.defineProperties(element, {
            value: {
                get: function () {
                    return this.au.controller.viewModel.value;
                },
                set: function (value) {
                    // aurelia binding converts "undefined" and "null" into empty string
                    // this does not translate well into "empty" menu items when several selects are bound to the same field
                    this.au.controller.viewModel.value = value === '' ? undefined : value;
                },
                configurable: true
            },
            options: {
                get: function () {
                    return this.au.controller.viewModel.root.querySelectorAll('.mdc-list-item');
                },
                configurable: true
            },
            selectedIndex: {
                get: function () {
                    return this.au.controller.viewModel.selectedIndex;
                },
                set: function (value) {
                    this.au.controller.viewModel.selectedIndex = value;
                },
                configurable: true
            },
            valid: {
                get: function () {
                    return this.au.controller.viewModel.valid;
                },
                set: function (value) {
                    this.au.controller.viewModel.valid = value;
                },
                configurable: true
            },
            addError: {
                value: function (error) {
                    this.au.controller.viewModel.addError(error);
                },
                configurable: true
            },
            removeError: {
                value: function (error) {
                    this.au.controller.viewModel.removeError(error);
                },
                configurable: true
            },
            renderErrors: {
                value: function () {
                    this.au.controller.viewModel.renderErrors();
                },
                configurable: true
            },
            focus: {
                value: function () {
                    this.au.controller.viewModel.focus();
                },
                configurable: true
            },
            blur: {
                value: function () {
                    this.au.controller.viewModel.blur();
                },
                configurable: true
            }
        });
    }
});
//# sourceMappingURL=mdc-select.js.map