import { __awaiter, __decorate, __metadata } from "tslib";
import { MdcComponent } from '@aurelia-mdc-web/base';
import { cssClasses, strings } from '@material/select';
import { inject, useView, customElement, processContent, children, bindingMode, TaskQueue } from 'aurelia-framework';
import { PLATFORM } from 'aurelia-pal';
import { mdcIconStrings } from './mdc-select-icon';
import { mdcHelperTextCssClasses } from './mdc-select-helper-text/mdc-select-helper-text';
import { bindable } from 'aurelia-typed-observable-plugin';
import { MDCSelectFoundationAurelia } from './mdc-select-foundation-aurelia';
import { MdcDefaultSelectConfiguration } from './mdc-default-select-configuration';
strings.CHANGE_EVENT = strings.CHANGE_EVENT.toLowerCase();
let selectId = 0;
/**
 * @selector mdc-select
 * @emits mdcselect:change | Emitted if user changed the value
 */
let MdcSelect = class MdcSelect extends MdcComponent {
    static processContent(_viewCompiler, _resources, element) {
        // move icon to the slot - this allows omitting slot specification
        const leadingIcon = element.querySelector(`[${mdcIconStrings.ATTRIBUTE}]`);
        leadingIcon === null || leadingIcon === void 0 ? void 0 : leadingIcon.setAttribute('slot', 'leading-icon');
        return true;
    }
    constructor(root, taskQueue, defaultConfiguration) {
        super(root);
        this.taskQueue = taskQueue;
        this.defaultConfiguration = defaultConfiguration;
        this.id = `mdc-select-${++selectId}`;
        this.errors = new Map();
        /** Styles the select as an outlined select */
        this.outlined = this.defaultConfiguration.outlined;
        defineMdcSelectElementApis(this.root);
    }
    labelChanged() {
        this.taskQueue.queueTask(() => { var _a; return (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.layout(); });
    }
    outlinedChanged() {
        this.taskQueue.queueTask(() => { var _a; return (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.layout(); });
    }
    requiredChanged() {
        return __awaiter(this, void 0, void 0, function* () {
            var _a, _b, _c;
            yield this.initialised;
            if (this.required) {
                (_a = this.selectAnchor) === null || _a === void 0 ? void 0 : _a.setAttribute('aria-required', 'true');
            }
            else {
                (_b = this.selectAnchor) === null || _b === void 0 ? void 0 : _b.removeAttribute('aria-required');
            }
            (_c = this.foundation) === null || _c === void 0 ? void 0 : _c.setRequired(this.required);
            this.taskQueue.queueTask(() => { var _a; return (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.layout(); });
        });
    }
    disabledChanged() {
        return __awaiter(this, void 0, void 0, function* () {
            var _a;
            yield this.initialised;
            (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.setDisabled(this.disabled);
        });
    }
    get value() {
        if (this.foundation) {
            return this.foundation.getValue();
        }
        else {
            return this._value;
        }
    }
    set value(value) {
        this.setValue(value);
    }
    setValue(value, skipNotify = false) {
        this._value = value;
        if (this.foundation) {
            this.foundation.setValue(value, skipNotify);
            this.foundation.layout();
        }
    }
    get valid() {
        var _a, _b;
        return (_b = (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.isValid()) !== null && _b !== void 0 ? _b : true;
    }
    set valid(value) {
        var _a;
        (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.setValid(value);
    }
    get selectedIndex() {
        return this.foundation.getSelectedIndex();
    }
    set selectedIndex(selectedIndex) {
        var _a;
        (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.setSelectedIndex(selectedIndex, /** closeMenu */ true);
    }
    addError(error) {
        this.errors.set(error, true);
        this.valid = false;
    }
    removeError(error) {
        this.errors.delete(error);
        this.valid = this.errors.size === 0;
    }
    renderErrors() {
        const helperText = this.root.nextElementSibling;
        if ((helperText === null || helperText === void 0 ? void 0 : helperText.tagName) === 'MDC-SELECT-HELPER-TEXT') {
            helperText.au.controller.viewModel.errors = Array.from(this.errors.keys())
                .filter(x => x.message !== null).map(x => x.message);
        }
    }
    initialise() {
        return __awaiter(this, void 0, void 0, function* () {
            var _a;
            const leadingIconEl = this.root.querySelector(`[${mdcIconStrings.ATTRIBUTE}]`);
            this.leadingIcon = leadingIconEl === null || leadingIconEl === void 0 ? void 0 : leadingIconEl.au['mdc-select-icon'].viewModel;
            const nextSibling = this.root.nextElementSibling;
            if ((nextSibling === null || nextSibling === void 0 ? void 0 : nextSibling.tagName) === mdcHelperTextCssClasses.ROOT.toUpperCase()) {
                this.helperText = nextSibling.au.controller.viewModel;
            }
            yield Promise.all([(_a = this.helperText) === null || _a === void 0 ? void 0 : _a.initialised, this.menu.initialised].filter(x => x));
            this.menu.list_.singleSelection = true;
        });
    }
    initialSyncWithDOM() {
        var _a, _b;
        // set initial value without emitting change events
        (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.setValue(this._value, true);
        (_b = this.foundation) === null || _b === void 0 ? void 0 : _b.layout();
        this.errors = new Map();
        this.valid = true;
    }
    getDefaultFoundation() {
        // DO NOT INLINE this variable. For backward compatibility, foundations take a Partial<MDCFooAdapter>.
        // To ensure we don't accidentally omit any methods, we need a separate, strongly typed adapter variable.
        const adapter = Object.assign(Object.assign(Object.assign(Object.assign({}, this.getSelectAdapterMethods()), this.getCommonAdapterMethods()), this.getOutlineAdapterMethods()), this.getLabelAdapterMethods());
        return new MDCSelectFoundationAurelia(adapter, this.getFoundationMap());
    }
    getSelectAdapterMethods() {
        return {
            setSelectedText: (text) => {
                this.selectedText.textContent = text;
            },
            isSelectAnchorFocused: () => document.activeElement === this.selectAnchor,
            getSelectAnchorAttr: (attr) => this.selectAnchor.getAttribute(attr),
            setSelectAnchorAttr: (attr, value) => {
                this.selectAnchor.setAttribute(attr, value);
            },
            removeSelectAnchorAttr: (attr) => {
                this.selectAnchor.removeAttribute(attr);
            },
            addMenuClass: (className) => {
                var _a;
                (_a = this.menuElement) === null || _a === void 0 ? void 0 : _a.classList.add(className);
            },
            removeMenuClass: (className) => {
                var _a;
                (_a = this.menuElement) === null || _a === void 0 ? void 0 : _a.classList.remove(className);
            },
            openMenu: () => {
                this.menu.open = true;
                this.menu.root.style.minWidth = this.menu.root.style.maxWidth = (this.hoistToBody || this.fixed) && !this.naturalWidth
                    ? `${this.root.clientWidth}px`
                    : '';
            },
            closeMenu: () => { this.menu.open = false; },
            getAnchorElement: () => this.root.querySelector(strings.SELECT_ANCHOR_SELECTOR),
            setMenuAnchorElement: (anchorEl) => {
                this.menu.anchor = anchorEl;
            },
            setMenuAnchorCorner: (anchorCorner) => {
                this.menu.setAnchorCorner(anchorCorner);
            },
            setMenuWrapFocus: (wrapFocus) => {
                this.menu.wrapFocus = wrapFocus;
            },
            getSelectedIndex: () => {
                const index = this.menu.selectedIndex;
                return index instanceof Array ? index[0] : index;
            },
            setSelectedIndex: (index) => {
                this.menu.selectedIndex = index;
            },
            removeAttributeAtIndex: (index, attributeName) => {
                this.menu.items[index].removeAttribute(attributeName);
            },
            focusMenuItemAtIndex: (index) => {
                this.menu.items[index].focus();
            },
            getMenuItemCount: () => this.menu.items.length,
            getMenuItemValues: () => this.menu.items.map(x => x.au.controller.viewModel.value),
            getMenuItemTextAtIndex: (index) => this.menu.getPrimaryTextAtIndex(index),
            isTypeaheadInProgress: () => this.menu.typeaheadInProgress,
            typeaheadMatchItem: (nextChar, startingIndex) => this.menu.typeaheadMatchItem(nextChar, startingIndex),
        };
    }
    getCommonAdapterMethods() {
        return {
            addClass: (className) => {
                this.root.classList.add(className);
            },
            removeClass: (className) => {
                this.root.classList.remove(className);
            },
            hasClass: (className) => this.root.classList.contains(className),
            setRippleCenter: (normalizedX) => { var _a; return (_a = this.lineRipple) === null || _a === void 0 ? void 0 : _a.setRippleCenter(normalizedX); },
            activateBottomLine: () => { var _a; return (_a = this.lineRipple) === null || _a === void 0 ? void 0 : _a.activate(); },
            deactivateBottomLine: () => { var _a; return (_a = this.lineRipple) === null || _a === void 0 ? void 0 : _a.deactivate(); },
            notifyChange: (value) => {
                const index = this.selectedIndex;
                this.emit(strings.CHANGE_EVENT, { value, index }, true /* shouldBubble  */);
                this.emit('change', { value, index }, true /* shouldBubble  */);
            },
        };
    }
    getOutlineAdapterMethods() {
        return {
            hasOutline: () => Boolean(this.outline),
            notchOutline: (labelWidth) => { var _a; return (_a = this.outline) === null || _a === void 0 ? void 0 : _a.notch(labelWidth); },
            closeOutline: () => { var _a; return (_a = this.outline) === null || _a === void 0 ? void 0 : _a.closeNotch(); },
        };
    }
    getLabelAdapterMethods() {
        return {
            hasLabel: () => !!this.mdcLabel,
            floatLabel: (shouldFloat) => { var _a; return (_a = this.mdcLabel) === null || _a === void 0 ? void 0 : _a.float(shouldFloat); },
            getLabelWidth: () => this.mdcLabel ? this.mdcLabel.getWidth() : 0,
            setLabelRequired: (isRequired) => { var _a; return (_a = this.mdcLabel) === null || _a === void 0 ? void 0 : _a.setRequired(isRequired); },
        };
    }
    handleChange() {
        var _a;
        (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleChange();
        this.emit('change', {}, true);
    }
    handleFocus() {
        var _a;
        (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleFocus();
    }
    handleBlur() {
        var _a;
        (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleBlur();
        // if class is set it means the menu is open,
        // do not emit blur since "conceptually" the element is still active
        if (!this.root.classList.contains(cssClasses.FOCUSED)) {
            this.emit('blur', {}, true);
        }
    }
    handleClick(evt) {
        var _a;
        this.selectAnchor.focus();
        (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleClick(this.getNormalizedXCoordinate(evt));
    }
    handleKeydown(evt) {
        var _a;
        (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleKeydown(evt);
        return true;
    }
    handleMenuItemAction(evt) {
        var _a;
        (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleMenuItemAction(evt.detail.index);
    }
    handleMenuOpened() {
        var _a;
        (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleMenuOpened();
    }
    handleMenuClosed() {
        var _a;
        (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.handleMenuClosed();
        if (!this.root.classList.contains(cssClasses.FOCUSED)) {
            this.emit('blur', {}, true);
        }
    }
    handleItemsChanged() {
        var _a, _b;
        (_a = this.foundation) === null || _a === void 0 ? void 0 : _a.layoutOptions();
        (_b = this.foundation) === null || _b === void 0 ? void 0 : _b.layout();
    }
    focus() {
        this.selectAnchor.focus();
    }
    blur() {
        this.selectAnchor.blur();
    }
    /**
     * @hidden
     * Calculates where the line ripple should start based on the x coordinate within the component.
     */
    getNormalizedXCoordinate(evt) {
        const targetClientRect = evt.target.getBoundingClientRect();
        const xCoordinate = this.isTouchEvent(evt) ? evt.touches[0].clientX : evt.clientX;
        return xCoordinate - targetClientRect.left;
    }
    isTouchEvent(evt) {
        return Boolean(evt.touches);
    }
    /**
     * @hidden
     * Returns a map of all subcomponents to subfoundations.
     */
    getFoundationMap() {
        var _a, _b;
        return {
            helperText: (_a = this.helperText) === null || _a === void 0 ? void 0 : _a.foundation,
            leadingIcon: (_b = this.leadingIcon) === null || _b === void 0 ? void 0 : _b.foundation
        };
    }
};
__decorate([
    children('mdc-list-items'),
    __metadata("design:type", Array)
], MdcSelect.prototype, "items", void 0);
__decorate([
    bindable.none,
    __metadata("design:type", String)
], MdcSelect.prototype, "label", void 0);
__decorate([
    bindable.booleanAttr,
    __metadata("design:type", Boolean)
], MdcSelect.prototype, "outlined", void 0);
__decorate([
    bindable.booleanAttr,
    __metadata("design:type", Boolean)
], MdcSelect.prototype, "required", void 0);
__decorate([
    bindable.booleanAttr,
    __metadata("design:type", Boolean)
], MdcSelect.prototype, "disabled", void 0);
__decorate([
    bindable.booleanAttr({ defaultBindingMode: bindingMode.oneTime }),
    __metadata("design:type", Boolean)
], MdcSelect.prototype, "hoistToBody", void 0);
__decorate([
    bindable.booleanAttr({ defaultBindingMode: bindingMode.oneTime }),
    __metadata("design:type", Boolean)
], MdcSelect.prototype, "fixed", void 0);
__decorate([
    bindable.none,
    __metadata("design:type", Object)
], MdcSelect.prototype, "anchorMargin", void 0);
__decorate([
    bindable.booleanAttr,
    __metadata("design:type", Boolean)
], MdcSelect.prototype, "naturalWidth", void 0);
MdcSelect = __decorate([
    inject(Element, TaskQueue, MdcDefaultSelectConfiguration),
    useView(PLATFORM.moduleName('./mdc-select.html')),
    customElement(cssClasses.ROOT),
    processContent(MdcSelect.processContent),
    __metadata("design:paramtypes", [HTMLElement, TaskQueue, MdcDefaultSelectConfiguration])
], MdcSelect);
export { MdcSelect };
function defineMdcSelectElementApis(element) {
    Object.defineProperties(element, {
        value: {
            get() {
                return this.au.controller.viewModel.value;
            },
            set(value) {
                // aurelia binding converts "undefined" and "null" into empty string
                // this does not translate well into "empty" menu items when several selects are bound to the same field
                this.au.controller.viewModel.value = value === '' ? undefined : value;
            },
            configurable: true
        },
        options: {
            get() {
                return this.au.controller.viewModel.root.querySelectorAll('.mdc-list-item');
            },
            configurable: true
        },
        selectedIndex: {
            get() {
                return this.au.controller.viewModel.selectedIndex;
            },
            set(value) {
                this.au.controller.viewModel.selectedIndex = value;
            },
            configurable: true
        },
        valid: {
            get() {
                return this.au.controller.viewModel.valid;
            },
            set(value) {
                this.au.controller.viewModel.valid = value;
            },
            configurable: true
        },
        addError: {
            value(error) {
                this.au.controller.viewModel.addError(error);
            },
            configurable: true
        },
        removeError: {
            value(error) {
                this.au.controller.viewModel.removeError(error);
            },
            configurable: true
        },
        renderErrors: {
            value() {
                this.au.controller.viewModel.renderErrors();
            },
            configurable: true
        },
        focus: {
            value() {
                this.au.controller.viewModel.focus();
            },
            configurable: true
        },
        blur: {
            value() {
                this.au.controller.viewModel.blur();
            },
            configurable: true
        }
    });
}
//# sourceMappingURL=mdc-select.js.map